"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpAdd = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
class OpAdd extends AbstractOp_1.AbstractOp {
    constructor(path, value) {
        super('add', path);
        this.value = value;
    }
    apply(doc) {
        const { val, key, obj } = json_pointer_1.find(doc, this.path);
        const value = this.value;
        if (!obj)
            doc = value;
        else if (typeof key === 'string')
            obj[key] = value;
        else {
            const length = obj.length;
            if (key < length)
                obj.splice(key, 0, this.value);
            else if (key > length)
                throw new Error('INVALID_INDEX');
            else
                obj.push(this.value);
        }
        return { doc, old: val };
    }
    toJson() {
        return {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            value: this.value,
        };
    }
    toPacked() {
        return [0, this.path, { v: this.value }];
    }
}
exports.OpAdd = OpAdd;
