"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpContains = void 0;
const AbstractPredicateOp_1 = require("./AbstractPredicateOp");
const json_pointer_1 = require("../../json-pointer");
class OpContains extends AbstractPredicateOp_1.AbstractPredicateOp {
    constructor(path, value, ignore_case) {
        super('contains', path);
        this.value = value;
        this.ignore_case = ignore_case;
    }
    test(doc) {
        const { val } = json_pointer_1.find(doc, this.path);
        if (typeof val !== 'string')
            return false;
        const testValue = this.ignore_case ? val.toLowerCase() : val;
        const testString = this.ignore_case ? this.value.toLowerCase() : this.value;
        const test = testValue.indexOf(testString) > -1;
        return test;
    }
    toJson() {
        const op = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            value: this.value,
        };
        if (this.ignore_case)
            op.ignore_case = this.ignore_case;
        return op;
    }
    toPacked() {
        const packed = [30, this.path, { v: this.value }];
        if (this.ignore_case)
            packed[2].i = 1;
        return packed;
    }
}
exports.OpContains = OpContains;
