"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpMerge = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
const util_1 = require("../util");
class OpMerge extends AbstractOp_1.AbstractOp {
    constructor(path, pos, props) {
        super('merge', path);
        this.pos = pos;
        this.props = props;
    }
    apply(doc) {
        const ref = json_pointer_1.find(doc, this.path);
        if (!json_pointer_1.isArrayReference(ref))
            throw new Error('INVALID_TARGET');
        if (ref.key <= 0)
            throw new Error('INVALID_KEY');
        const one = ref.obj[ref.key - 1];
        const two = ref.obj[ref.key];
        const merged = this.merge(one, two);
        ref.obj[ref.key - 1] = merged;
        ref.obj.splice(ref.key, 1);
        return { doc, old: [one, two] };
    }
    merge(one, two) {
        if (typeof one === 'string' && typeof two === 'string')
            return one + two;
        if (typeof one === 'number' && typeof two === 'number')
            return one + two;
        if (util_1.isTextNode(one) && util_1.isTextNode(two))
            return Object.assign(Object.assign(Object.assign({}, one), two), { text: one.text + two.text });
        if (util_1.isElementNode(one) && util_1.isElementNode(two))
            return Object.assign(Object.assign(Object.assign({}, one), two), { children: [...one.children, ...two.children] });
        return [one, two];
    }
    toJson() {
        const op = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            pos: this.pos,
        };
        if (this.props)
            op.props = this.props;
        return op;
    }
    toPacked() {
        const packed = [11, this.path, { i: this.pos }];
        if (this.props)
            packed[2].p = this.props;
        return packed;
    }
}
exports.OpMerge = OpMerge;
