"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpRemove = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
class OpRemove extends AbstractOp_1.AbstractOp {
    constructor(path, oldValue) {
        super('remove', path);
        this.oldValue = oldValue;
    }
    apply(doc) {
        const ref = json_pointer_1.find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        if (json_pointer_1.isObjectReference(ref))
            delete ref.obj[ref.key];
        else if (json_pointer_1.isArrayReference(ref)) {
            if (ref.val !== undefined)
                ref.obj.splice(ref.key, 1);
        }
        else
            doc = null;
        return { doc, old: ref.val };
    }
    toJson() {
        const json = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
        };
        if (this.oldValue !== undefined)
            json.oldValue = this.oldValue;
        return json;
    }
    toPacked() {
        const packed = this.oldValue !== undefined ? [1, this.path] : [1, this.path, { o: this.oldValue }];
        return packed;
    }
}
exports.OpRemove = OpRemove;
