"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpReplace = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
class OpReplace extends AbstractOp_1.AbstractOp {
    constructor(path, value, oldValue) {
        super('replace', path);
        this.value = value;
        this.oldValue = oldValue;
    }
    apply(doc) {
        const ref = json_pointer_1.find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        if (json_pointer_1.isObjectReference(ref))
            ref.obj[ref.key] = this.value;
        else if (json_pointer_1.isArrayReference(ref))
            ref.obj[ref.key] = this.value;
        else
            doc = this.value;
        return { doc, old: ref.val };
    }
    toJson() {
        const json = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            value: this.value,
        };
        if (this.oldValue !== undefined)
            json.oldValue = this.oldValue;
        return json;
    }
    toPacked() {
        const packed = [2, this.path, { v: this.value }];
        if (this.oldValue !== undefined)
            packed[2].o = this.oldValue;
        return packed;
    }
}
exports.OpReplace = OpReplace;
