"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpStrIns = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
class OpStrIns extends AbstractOp_1.AbstractOp {
    constructor(path, pos, str) {
        super('str_ins', path);
        this.pos = pos;
        this.str = str;
    }
    apply(doc) {
        const { val, key, obj } = json_pointer_1.find(doc, this.path);
        if (typeof val !== 'string') {
            if (val !== undefined)
                throw new Error('NOT_A_STRING');
            if (this.pos !== 0)
                throw new Error('POS');
        }
        const str = typeof val === 'string' ? val : '';
        const pos = Math.min(this.pos, str.length);
        const before = str.slice(0, pos);
        const after = str.slice(pos);
        const result = before + this.str + after;
        if (obj)
            obj[key] = result;
        else
            doc = result;
        return { doc, old: val };
    }
    toJson() {
        const op = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            pos: this.pos,
            str: this.str,
        };
        return op;
    }
    toPacked() {
        const packed = [6, this.path, { i: this.pos, s: this.str }];
        return packed;
    }
}
exports.OpStrIns = OpStrIns;
