"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpTestString = void 0;
const AbstractPredicateOp_1 = require("./AbstractPredicateOp");
const json_pointer_1 = require("../../json-pointer");
class OpTestString extends AbstractPredicateOp_1.AbstractPredicateOp {
    constructor(path, pos, str, not) {
        super('test_string', path);
        this.pos = pos;
        this.str = str;
        this.not = not;
    }
    test(doc) {
        const { val } = json_pointer_1.find(doc, this.path);
        if (typeof val !== 'string')
            return false;
        const length = val.length;
        const start = Math.min(this.pos, length);
        const end = Math.min(this.pos + this.str.length, length);
        const test = val.substring(start, end) === this.str;
        return this.not ? !test : test;
    }
    toJson() {
        const op = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            pos: this.pos,
            str: this.str,
        };
        if (this.not)
            op.not = this.not;
        return op;
    }
    toPacked() {
        const packed = [40, this.path, { i: this.pos, s: this.str }];
        if (this.not)
            packed[2].n = 1;
        return packed;
    }
}
exports.OpTestString = OpTestString;
