"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpcRxServer = void 0;
const tslib_1 = require("tslib");
const util_1 = require("./util");
class JsonRpcRxServer {
    constructor({ call, notify }) {
        this.call = call;
        this.notify = notify;
    }
    onSubscribe(ctx, message) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [id, name, payload] = message;
            util_1.assertId(id);
            util_1.assertName(name);
            const result = yield this.call(name, ctx, payload);
            return [0, id, result];
        });
    }
    onNotification(ctx, message) {
        const [name, payload] = message;
        util_1.assertName(name);
        this.notify(name, ctx, payload);
    }
    onMessage(ctx, message) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                if (!util_1.isArray(message))
                    throw new Error('Invalid message');
                if (!message.length)
                    return null;
                if (util_1.isArray(message[0]))
                    return yield Promise.all(message.map(msg => this.onMessage(ctx, msg)));
                const [one] = message;
                if (typeof one === 'string') {
                    this.onNotification(ctx, message);
                    return null;
                }
                if (one > 0)
                    return yield this.onSubscribe(ctx, message);
                throw new Error('Invalid message');
            }
            catch (error) {
                const id = util_1.isArray(message) && (typeof message[0] === 'number') && (message[0] > 0) && (Math.round(message[0]) === message[0])
                    ? message[0] : -1;
                return [-1, id, { message: error instanceof Error ? error.message : String(error) }];
            }
        });
    }
}
exports.JsonRpcRxServer = JsonRpcRxServer;
