import { AbstractOp } from './AbstractOp';
import { find, formatJsonPointer } from '../../json-pointer';
export class OpAdd extends AbstractOp {
    constructor(path, value) {
        super('add', path);
        this.value = value;
    }
    apply(doc) {
        const { val, key, obj } = find(doc, this.path);
        const value = this.value;
        if (!obj)
            doc = value;
        else if (typeof key === 'string')
            obj[key] = value;
        else {
            const length = obj.length;
            if (key < length)
                obj.splice(key, 0, this.value);
            else if (key > length)
                throw new Error('INVALID_INDEX');
            else
                obj.push(this.value);
        }
        return { doc, old: val };
    }
    toJson() {
        return {
            op: this.op,
            path: formatJsonPointer(this.path),
            value: this.value,
        };
    }
    toPacked() {
        return [0, this.path, { v: this.value }];
    }
}
