import { AbstractOp } from './AbstractOp';
import { find, isArrayReference, isObjectReference, formatJsonPointer } from '../../json-pointer';
const { isArray } = Array;
export class OpExtend extends AbstractOp {
    constructor(path, props, deleteNull) {
        super('extend', path);
        this.props = props;
        this.deleteNull = deleteNull;
    }
    apply(doc) {
        const ref = find(doc, this.path);
        if (isArrayReference(ref)) {
            if (ref.val !== undefined) {
                ref.obj[ref.key] = this.extend(ref.val);
            }
        }
        else if (isObjectReference(ref)) {
            ref.obj[ref.key] = this.extend(ref.val);
        }
        else {
            doc = this.extend(doc);
        }
        return { doc };
    }
    extend(value) {
        if (isArray(value))
            return value;
        if (typeof value !== 'object')
            return value;
        if (!value)
            return value;
        for (const [key, v] of Object.entries(this.props)) {
            if (key === '__proto__')
                throw new Error('NO_PROTO');
            if (v === null && this.deleteNull) {
                delete value[key];
                continue;
            }
            value[key] = v;
        }
        return value;
    }
    toJson() {
        const op = {
            op: this.op,
            path: formatJsonPointer(this.path),
            props: this.props,
        };
        if (this.deleteNull)
            op.deleteNull = this.deleteNull;
        return op;
    }
    toPacked() {
        const packed = [12, this.path, { p: this.props }];
        if (this.deleteNull)
            packed[2].d = 1;
        return packed;
    }
}
