import { AbstractOp } from './AbstractOp';
import { find, isArrayReference, formatJsonPointer } from '../../json-pointer';
import { isTextNode, isElementNode } from '../util';
export class OpMerge extends AbstractOp {
    constructor(path, pos, props) {
        super('merge', path);
        this.pos = pos;
        this.props = props;
    }
    apply(doc) {
        const ref = find(doc, this.path);
        if (!isArrayReference(ref))
            throw new Error('INVALID_TARGET');
        if (ref.key <= 0)
            throw new Error('INVALID_KEY');
        const one = ref.obj[ref.key - 1];
        const two = ref.obj[ref.key];
        const merged = this.merge(one, two);
        ref.obj[ref.key - 1] = merged;
        ref.obj.splice(ref.key, 1);
        return { doc, old: [one, two] };
    }
    merge(one, two) {
        if (typeof one === 'string' && typeof two === 'string')
            return one + two;
        if (typeof one === 'number' && typeof two === 'number')
            return one + two;
        if (isTextNode(one) && isTextNode(two))
            return { ...one, ...two, text: one.text + two.text };
        if (isElementNode(one) && isElementNode(two))
            return { ...one, ...two, children: [...one.children, ...two.children] };
        return [one, two];
    }
    toJson() {
        const op = {
            op: this.op,
            path: formatJsonPointer(this.path),
            pos: this.pos,
        };
        if (this.props)
            op.props = this.props;
        return op;
    }
    toPacked() {
        const packed = [11, this.path, { i: this.pos }];
        if (this.props)
            packed[2].p = this.props;
        return packed;
    }
}
