import { AbstractOp } from './AbstractOp';
import { find, isObjectReference, isArrayReference, formatJsonPointer } from '../../json-pointer';
export class OpRemove extends AbstractOp {
    constructor(path, oldValue) {
        super('remove', path);
        this.oldValue = oldValue;
    }
    apply(doc) {
        const ref = find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        if (isObjectReference(ref))
            delete ref.obj[ref.key];
        else if (isArrayReference(ref)) {
            if (ref.val !== undefined)
                ref.obj.splice(ref.key, 1);
        }
        else
            doc = null;
        return { doc, old: ref.val };
    }
    toJson() {
        const json = {
            op: this.op,
            path: formatJsonPointer(this.path),
        };
        if (this.oldValue !== undefined)
            json.oldValue = this.oldValue;
        return json;
    }
    toPacked() {
        const packed = this.oldValue !== undefined ? [1, this.path] : [1, this.path, { o: this.oldValue }];
        return packed;
    }
}
