import { AbstractOp } from './AbstractOp';
import { find, isObjectReference, isArrayReference, formatJsonPointer } from '../../json-pointer';
export class OpReplace extends AbstractOp {
    constructor(path, value, oldValue) {
        super('replace', path);
        this.value = value;
        this.oldValue = oldValue;
    }
    apply(doc) {
        const ref = find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        if (isObjectReference(ref))
            ref.obj[ref.key] = this.value;
        else if (isArrayReference(ref))
            ref.obj[ref.key] = this.value;
        else
            doc = this.value;
        return { doc, old: ref.val };
    }
    toJson() {
        const json = {
            op: this.op,
            path: formatJsonPointer(this.path),
            value: this.value,
        };
        if (this.oldValue !== undefined)
            json.oldValue = this.oldValue;
        return json;
    }
    toPacked() {
        const packed = [2, this.path, { v: this.value }];
        if (this.oldValue !== undefined)
            packed[2].o = this.oldValue;
        return packed;
    }
}
