import { AbstractOp } from './AbstractOp';
import { find, formatJsonPointer } from '../../json-pointer';
export class OpStrDel extends AbstractOp {
    constructor(path, pos, str, len) {
        super('str_del', path);
        this.pos = pos;
        this.str = str;
        this.len = len;
    }
    deleteLength() {
        return typeof this.str === 'string' ? this.str.length : this.len;
    }
    apply(doc) {
        const { val, key, obj } = find(doc, this.path);
        if (typeof val !== 'string')
            throw new Error('NOT_A_STRING');
        const length = val.length;
        const pos = Math.min(this.pos, val.length);
        const start = Math.min(pos, length);
        const deletionLength = this.str !== undefined ? this.str.length : this.len;
        const end = Math.min(pos + deletionLength, length);
        const before = val.slice(0, start);
        const after = val.substr(end);
        const result = before + after;
        if (obj)
            obj[key] = result;
        else
            doc = result;
        return { doc, old: val };
    }
    toJson() {
        if (typeof this.str === 'string') {
            return {
                op: this.op,
                path: formatJsonPointer(this.path),
                pos: this.pos,
                str: this.str,
            };
        }
        return {
            op: this.op,
            path: formatJsonPointer(this.path),
            pos: this.pos,
            len: this.len,
        };
    }
    toPacked() {
        const packed = typeof this.str === 'string'
            ? [7, this.path, { i: this.pos, s: this.str }]
            : [7, this.path, { i: this.pos, l: this.len }];
        return packed;
    }
}
