import { AbstractPredicateOp } from './AbstractPredicateOp';
import { find, formatJsonPointer } from '../../json-pointer';
export class OpTestString extends AbstractPredicateOp {
    constructor(path, pos, str, not) {
        super('test_string', path);
        this.pos = pos;
        this.str = str;
        this.not = not;
    }
    test(doc) {
        const { val } = find(doc, this.path);
        if (typeof val !== 'string')
            return false;
        const length = val.length;
        const start = Math.min(this.pos, length);
        const end = Math.min(this.pos + this.str.length, length);
        const test = val.substring(start, end) === this.str;
        return this.not ? !test : test;
    }
    toJson() {
        const op = {
            op: this.op,
            path: formatJsonPointer(this.path),
            pos: this.pos,
            str: this.str,
        };
        if (this.not)
            op.not = this.not;
        return op;
    }
    toPacked() {
        const packed = [40, this.path, { i: this.pos, s: this.str }];
        if (this.not)
            packed[2].n = 1;
        return packed;
    }
}
