"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpExtend = void 0;
var tslib_1 = require("tslib");
var AbstractOp_1 = require("./AbstractOp");
var json_pointer_1 = require("../../json-pointer");
var isArray = Array.isArray;
var OpExtend = (function (_super) {
    tslib_1.__extends(OpExtend, _super);
    function OpExtend(path, props, deleteNull) {
        var _this = _super.call(this, 'extend', path) || this;
        _this.props = props;
        _this.deleteNull = deleteNull;
        return _this;
    }
    OpExtend.prototype.apply = function (doc) {
        var ref = json_pointer_1.find(doc, this.path);
        if (json_pointer_1.isArrayReference(ref)) {
            if (ref.val !== undefined) {
                ref.obj[ref.key] = this.extend(ref.val);
            }
        }
        else if (json_pointer_1.isObjectReference(ref)) {
            ref.obj[ref.key] = this.extend(ref.val);
        }
        else {
            doc = this.extend(doc);
        }
        return { doc: doc };
    };
    OpExtend.prototype.extend = function (value) {
        var e_1, _a;
        if (isArray(value))
            return value;
        if (typeof value !== 'object')
            return value;
        if (!value)
            return value;
        try {
            for (var _b = tslib_1.__values(Object.entries(this.props)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], v = _d[1];
                if (key === '__proto__')
                    throw new Error('NO_PROTO');
                if (v === null && this.deleteNull) {
                    delete value[key];
                    continue;
                }
                value[key] = v;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return value;
    };
    OpExtend.prototype.toJson = function () {
        var op = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            props: this.props,
        };
        if (this.deleteNull)
            op.deleteNull = this.deleteNull;
        return op;
    };
    OpExtend.prototype.toPacked = function () {
        var packed = [12, this.path, { p: this.props }];
        if (this.deleteNull)
            packed[2].d = 1;
        return packed;
    };
    return OpExtend;
}(AbstractOp_1.AbstractOp));
exports.OpExtend = OpExtend;
