"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpRemove = void 0;
var tslib_1 = require("tslib");
var AbstractOp_1 = require("./AbstractOp");
var json_pointer_1 = require("../../json-pointer");
var OpRemove = (function (_super) {
    tslib_1.__extends(OpRemove, _super);
    function OpRemove(path, oldValue) {
        var _this = _super.call(this, 'remove', path) || this;
        _this.oldValue = oldValue;
        return _this;
    }
    OpRemove.prototype.apply = function (doc) {
        var ref = json_pointer_1.find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        if (json_pointer_1.isObjectReference(ref))
            delete ref.obj[ref.key];
        else if (json_pointer_1.isArrayReference(ref)) {
            if (ref.val !== undefined)
                ref.obj.splice(ref.key, 1);
        }
        else
            doc = null;
        return { doc: doc, old: ref.val };
    };
    OpRemove.prototype.toJson = function () {
        var json = {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
        };
        if (this.oldValue !== undefined)
            json.oldValue = this.oldValue;
        return json;
    };
    OpRemove.prototype.toPacked = function () {
        var packed = this.oldValue !== undefined ? [1, this.path] : [1, this.path, { o: this.oldValue }];
        return packed;
    };
    return OpRemove;
}(AbstractOp_1.AbstractOp));
exports.OpRemove = OpRemove;
