"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpStrDel = void 0;
var tslib_1 = require("tslib");
var AbstractOp_1 = require("./AbstractOp");
var json_pointer_1 = require("../../json-pointer");
var OpStrDel = (function (_super) {
    tslib_1.__extends(OpStrDel, _super);
    function OpStrDel(path, pos, str, len) {
        var _this = _super.call(this, 'str_del', path) || this;
        _this.pos = pos;
        _this.str = str;
        _this.len = len;
        return _this;
    }
    OpStrDel.prototype.deleteLength = function () {
        return typeof this.str === 'string' ? this.str.length : this.len;
    };
    OpStrDel.prototype.apply = function (doc) {
        var _a = json_pointer_1.find(doc, this.path), val = _a.val, key = _a.key, obj = _a.obj;
        if (typeof val !== 'string')
            throw new Error('NOT_A_STRING');
        var length = val.length;
        var pos = Math.min(this.pos, val.length);
        var start = Math.min(pos, length);
        var deletionLength = this.str !== undefined ? this.str.length : this.len;
        var end = Math.min(pos + deletionLength, length);
        var before = val.slice(0, start);
        var after = val.substr(end);
        var result = before + after;
        if (obj)
            obj[key] = result;
        else
            doc = result;
        return { doc: doc, old: val };
    };
    OpStrDel.prototype.toJson = function () {
        if (typeof this.str === 'string') {
            return {
                op: this.op,
                path: json_pointer_1.formatJsonPointer(this.path),
                pos: this.pos,
                str: this.str,
            };
        }
        return {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            pos: this.pos,
            len: this.len,
        };
    };
    OpStrDel.prototype.toPacked = function () {
        var packed = typeof this.str === 'string'
            ? [7, this.path, { i: this.pos, s: this.str }]
            : [7, this.path, { i: this.pos, l: this.len }];
        return packed;
    };
    return OpStrDel;
}(AbstractOp_1.AbstractOp));
exports.OpStrDel = OpStrDel;
