"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpcServer = void 0;
var tslib_1 = require("tslib");
var JsonRpcRxServer_1 = require("../json-rx/JsonRpcRxServer");
var util_1 = require("../json-rx/util");
var JsonRpcServer = (function () {
    function JsonRpcServer(params) {
        this.rx = new JsonRpcRxServer_1.JsonRpcRxServer(params);
        this.strict = params.strict || false;
    }
    JsonRpcServer.prototype.formatError = function (error, id) {
        if (id === void 0) { id = null; }
        var message = {
            id: id,
            error: error,
        };
        if (this.strict)
            message.jsonrpc = '2.0';
        return message;
    };
    JsonRpcServer.prototype.onMessage = function (ctx, message) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var id, isNotification, result, _a, type, responseMessage, error_1, rpcError;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        id = null;
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        if (!message || (typeof message !== 'object') || util_1.isArray(message))
                            return [2, this.formatError({ message: 'Invalid message.', code: 0 }, id)];
                        isNotification = (typeof message.id !== 'number') && (typeof message.id !== 'string');
                        if (!isNotification)
                            id = message.id;
                        if (message.method === undefined)
                            return [2, this.formatError({ message: 'Method not specified.', code: 0 }, id)];
                        if (typeof message.method !== 'string')
                            return [2, this.formatError({ message: 'Invalid method.', code: 0 }, id)];
                        if (this.strict && message.jsonrpc !== '2.0')
                            return [2, this.formatError({ message: 'Only JSON-RPC version 2.0 is supported.', code: 0 }, id)];
                        if (isNotification) {
                            this.rx.onMessage(ctx, [message.method, message.params]);
                            return [2, null];
                        }
                        return [4, this.rx.onMessage(ctx, [1, message.method, message.params])];
                    case 2:
                        result = _b.sent();
                        if (!result)
                            return [2, null];
                        _a = tslib_1.__read(result, 1), type = _a[0];
                        if (type === -1)
                            throw result[2];
                        responseMessage = {
                            id: id,
                            result: result[2],
                        };
                        if (this.strict)
                            responseMessage.jsonrpc = '2.0';
                        return [2, responseMessage];
                    case 3:
                        error_1 = _b.sent();
                        rpcError = {
                            code: (!!error_1 && (typeof error_1 === 'object') && (typeof error_1.code === 'number'))
                                ? error_1.code
                                : 0,
                            message: error_1 instanceof Error
                                ? error_1.message
                                : (!!error_1 && (typeof error_1 === 'object'))
                                    ? String(error_1.message)
                                    : String(error_1),
                        };
                        if (!!error_1 && (typeof error_1 === 'object') && (error_1.data !== undefined))
                            rpcError.data = error_1.data;
                        return [2, this.formatError(rpcError, id)];
                    case 4: return [2];
                }
            });
        });
    };
    return JsonRpcServer;
}());
exports.JsonRpcServer = JsonRpcServer;
