"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRxClient = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var util_1 = require("./util");
var TimedQueue_1 = require("./TimedQueue");
var JsonRxClient = (function () {
    function JsonRxClient(_a) {
        var send = _a.send, _b = _a.bufferSize, bufferSize = _b === void 0 ? 100 : _b, _c = _a.bufferTime, bufferTime = _c === void 0 ? 10 : _c;
        this.cnt = 1;
        this.observers = new Map();
        this.buffer = new TimedQueue_1.TimedQueue();
        this.buffer.itemLimit = bufferSize;
        this.buffer.timeLimit = bufferTime;
        this.buffer.onFlush = function (messages) {
            send(messages.length === 1 ? messages[0] : messages);
        };
    }
    JsonRxClient.prototype.sendSubscribe = function (message) {
        this.buffer.push(message);
    };
    JsonRxClient.prototype.sendUnsubscribe = function (id) {
        this.buffer.push([-3, id]);
    };
    JsonRxClient.prototype.onData = function (message) {
        var _a = tslib_1.__read(message, 3), id = _a[1], payload = _a[2];
        util_1.assertId(id);
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.next(payload);
    };
    JsonRxClient.prototype.onComplete = function (message) {
        var _a = tslib_1.__read(message, 3), id = _a[1], payload = _a[2];
        util_1.assertId(id);
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        if (payload !== undefined)
            observer.observer.next(payload);
        observer.observer.complete();
    };
    JsonRxClient.prototype.onError = function (message) {
        var _a = tslib_1.__read(message, 3), id = _a[1], error = _a[2];
        util_1.assertId(id);
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        observer.observer.error(error);
    };
    JsonRxClient.prototype.onMessage = function (message) {
        if (!util_1.isArray(message))
            throw new Error('Invalid message');
        var _a = tslib_1.__read(message, 1), type = _a[0];
        if (util_1.isArray(type)) {
            for (var i = 0; i < message.length; i++)
                this.onMessage(message[i]);
            return;
        }
        if (type === 0)
            return this.onComplete(message);
        if (type === -1)
            return this.onError(message);
        if (type === -2)
            return this.onData(message);
        throw new Error('Invalid message');
    };
    JsonRxClient.prototype.call = function (method, payload) {
        var _this = this;
        var id = this.cnt++;
        var observable = new rxjs_1.Observable(function (observer) {
            var entry = { observer: observer };
            _this.observers.set(id, entry);
            return function () {
                _this.observers.delete(id);
                if (!entry.completed)
                    _this.sendUnsubscribe(id);
            };
        });
        this.sendSubscribe([id, method, payload]);
        return observable;
    };
    JsonRxClient.prototype.notify = function (name, payload) {
        var message = payload !== undefined ? [name, payload] : [name];
        this.buffer.push(message);
    };
    JsonRxClient.prototype.stop = function () {
        var e_1, _a;
        this.buffer.onFlush = function (message) { };
        try {
            for (var _b = tslib_1.__values(this.observers.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var observer = _c.value.observer;
                observer.complete();
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    return JsonRxClient;
}());
exports.JsonRxClient = JsonRxClient;
